"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageCollector = void 0;
const NodeError_1 = require("./NodeError");
class MessageCollector {
    constructor() {
        this.errors = [];
    }
    error(error, position) {
        if (error instanceof NodeError_1.PositionCapableError) {
            if (!this.errors.some($ => $.message === error.message && (0, NodeError_1.isSamePosition)($.position, error.position))) {
                this.errors.push(error);
            }
        }
        else {
            const message = error instanceof Error ? error.message : error;
            if (position) {
                if (!this.errors.some($ => $.message === message && (0, NodeError_1.isSamePosition)($.position, position))) {
                    const err = new NodeError_1.PositionCapableError(message, position);
                    if (error instanceof Error && error.stack) {
                        err.stack = error.stack;
                    }
                    this.errors.push(err);
                }
            }
            else {
                const err = error instanceof Error ? error : new Error(error);
                this.errors.push(err);
                if (error instanceof Error && error.stack) {
                    err.stack = error.stack;
                }
            }
        }
    }
    errorIfBranchDoesntHaveAny(message, node) {
        const position = (message instanceof NodeError_1.PositionCapableError && message.position) || (node && node.astNode);
        if (!this.hasErrorForBranch(position)) {
            this.error(message, position);
        }
    }
    warning(message, node) {
        if (!this.errors.some($ => $.message === message && (0, NodeError_1.isSamePosition)($.position, node.astNode))) {
            this.errors.push(new NodeError_1.LysCompilerError(message, node, true));
        }
    }
    removeErrorsFromModule(moduleName) {
        this.errors = this.errors.filter($ => $.position && $.position.moduleName !== moduleName);
    }
    hasErrorForBranch(position) {
        return this.errors.some($ => (0, NodeError_1.isSamePositionOrInside)(position, $.position) && !$.warning);
    }
    hasErrors() {
        return this.errors.some($ => !$.warning);
    }
    /**
     * Takes the content of other message collector into this instance. The other instance will be purged.
     * @param otherMessageCollector
     */
    mergeWith(otherMessageCollector) {
        otherMessageCollector.errors.forEach(error => this.error(error));
        otherMessageCollector.errors.length = 0;
    }
}
exports.MessageCollector = MessageCollector;
//# sourceMappingURL=MessageCollector.js.map