import { Nodes } from './nodes';
import { Type, IntersectionType } from './types';
export declare const DEBUG_TYPES: boolean;
export interface IPositionCapable {
    readonly start: number;
    readonly end: number;
    readonly moduleName: string;
}
export declare type IErrorPositionCapable = {
    readonly message: string;
    readonly node?: Nodes.Node;
    readonly warning?: boolean;
    readonly position: IPositionCapable;
} & Error;
export declare function isSamePosition(lhs: IPositionCapable, rhs: IPositionCapable): boolean;
export declare function isSamePositionOrInside(parent: IPositionCapable, child: IPositionCapable): boolean;
export declare function printDebugType(type?: Type | null | undefined | void): string;
export declare class PositionCapableError extends Error implements IErrorPositionCapable {
    message: string;
    readonly position: IPositionCapable;
    warning: boolean;
    constructor(message: string, position: IPositionCapable, warning?: boolean);
}
export declare abstract class AstNodeError extends PositionCapableError implements IErrorPositionCapable {
    message: string;
    node: Nodes.Node;
    warning: boolean;
    constructor(message: string, node: Nodes.Node, warning?: boolean);
    private static ensureNodePosition;
}
export declare class LysScopeError extends AstNodeError {
}
export declare class LysTypeError extends AstNodeError {
}
export declare class LysSemanticError extends AstNodeError {
}
export declare class LysCompilerError extends AstNodeError {
}
export declare class TypeMismatch extends LysTypeError {
    givenType: Type;
    expectedType: Type;
    constructor(givenType: Type, expectedType: Type, node: Nodes.Node);
}
export declare class CannotInferReturnType extends LysTypeError {
    constructor(node: Nodes.Node);
}
export declare class NotAValidType extends LysTypeError {
    constructor(node: Nodes.Node, type: Type | null);
}
export declare class UnexpectedType extends LysTypeError {
    type: Type;
    constructor(type: Type, node: Nodes.Node);
}
export declare class InvalidOverload extends LysTypeError {
    functionType: IntersectionType;
    givenTypes: Type[];
    constructor(functionType: IntersectionType, givenTypes: Type[], node: Nodes.Node);
}
export declare class InvalidCall extends LysTypeError {
    expectedTypes: Type[];
    givenTypes: Type[];
    constructor(expectedTypes: Type[], givenTypes: Type[], node: Nodes.Node);
}
export declare class UnreachableCode extends LysSemanticError {
    constructor(node: Nodes.Node);
}
export declare class NotAFunction extends LysTypeError {
    type: Type;
    constructor(type: Type, node: Nodes.Node);
}
