import { MessageCollector } from './MessageCollector';
import { System } from './System';
import { Nodes, PhaseFlags } from './nodes';
export declare class ParsingContext {
    system: System;
    messageCollector: MessageCollector;
    paths: string[];
    modulesInContext: Map<string, Nodes.DocumentNode>;
    functionsInTable: Set<string>;
    signatures: Map<string, any>;
    private typeNumbers;
    constructor(system: System);
    reset(): void;
    addFunctionToTable(funInternalIdentifier: string): void;
    getOrderedTable(): string[];
    getFunctionInTableNumber(funInternalIdentifier: string): number;
    /**
     * Invalidates a module and it's dependencies. This function should be called
     * after modification of any source file.
     * @param moduleName
     */
    invalidateModule(moduleName: string): void;
    /**
     * Gets an internal identifier used for compilation time. It returns a unique
     * name every time.
     * @param moduleName
     * @param name
     */
    getInternalName(moduleName: string, name: string): string;
    /**
     * Returns a type discriminant number for the specified name
     * @param moduleName
     * @param typeName
     */
    getTypeDiscriminant(moduleName: string, typeName: string): number;
    /**
     * Gets the internal representation of the desired moduleName if exists.
     * @param moduleName
     */
    getExistingParsingPhaseForModule(moduleName: string): Nodes.DocumentNode | undefined;
    /**
     * Loads a module using a moduleName if exists.
     * @param moduleName
     */
    getParsingPhaseForModule(moduleName: string): Nodes.DocumentNode;
    /**
     * Loads a module into the context using `fileName`, `moduleName` and `content`
     * @param fileName
     * @param moduleName
     * @param content
     */
    getParsingPhaseForContent(fileName: string, moduleName: string, content: string): Nodes.DocumentNode;
    getPhase(moduleName: string, phase: PhaseFlags, debug?: boolean): Nodes.DocumentNode;
    /**
     * Obtains the fully qyalified name of a file.
     * @param file
     */
    getModuleFQNForFile(file: string): string;
    outline(): string;
    /**
     * Gets or loads a module by fileName & moduleName.
     *
     * If the module exists in the current context it gets it. Otherwise it is
     * loaded.
     *
     * @param fileName
     * @param moduleName
     */
    private getParsingPhaseForFile;
    /**
     * Resolves a file in the context's path list
     * @param file
     */
    private resolveFileInPath;
    /**
     * Tries to resolve a fileName using a moduleName as input.
     * @param moduleName
     */
    private resolveModule;
}
