import { Local, Nodes, Global } from './nodes';
import { FunctionSignatureType } from './types';
export declare abstract class Annotation {
    get name(): string;
    toString(): string;
}
export declare type IAnnotationConstructor<T extends Annotation> = {
    new (...args: any[]): T;
};
export declare namespace annotations {
    class IsTailRec extends Annotation {
    }
    class Injected extends Annotation {
    }
    class IsOverloaded extends Annotation {
    }
    class Inline extends Annotation {
    }
    class Getter extends Annotation {
    }
    class Setter extends Annotation {
    }
    class Method extends Annotation {
    }
    class MethodCall extends Annotation {
    }
    class SignatureDeclaration extends Annotation {
    }
    class Explicit extends Annotation {
    }
    class ByPassFunction extends Annotation {
    }
    class Extern extends Annotation {
        module: string;
        fn: string;
        constructor(module: string, fn: string);
    }
    class Export extends Annotation {
        exportedName: string;
        constructor(exportedName: string);
    }
    class MutableDeclaration extends Annotation {
    }
    class LabelId extends Annotation {
        label: string;
        constructor(label: string);
    }
    class FunctionInTable extends Annotation {
        nameIdentifier: Nodes.NameIdentifierNode;
        constructor(nameIdentifier: Nodes.NameIdentifierNode);
        toString(): string;
    }
    class LocalIdentifier extends Annotation {
        local: Local | Global;
        constructor(local: Local | Global);
        toString(): string;
    }
    class IsTailRecCall extends Annotation {
    }
    class CurrentLoop extends Annotation {
        loop: Nodes.LoopNode;
        constructor(loop: Nodes.LoopNode);
    }
    class ImplicitCall extends Annotation {
        implicitCall: Nodes.InjectedFunctionCallNode;
        constructor(implicitCall: Nodes.InjectedFunctionCallNode);
        toString(): string;
    }
    class IsFunctionReference extends Annotation {
        fun: FunctionSignatureType;
        constructor(fun: FunctionSignatureType);
    }
    class IsUnreachable extends Annotation {
    }
    class IsValueNode extends Annotation {
    }
    class IsAssignationLHS extends Annotation {
    }
    class IsTypeNode extends Annotation {
    }
    class NoStd extends Annotation {
    }
    class IsReturnExpression extends Annotation {
        targetLocal: null | Local;
        toString(): string;
    }
}
