"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.annotations = exports.Annotation = void 0;
const nodes_1 = require("./nodes");
class Annotation {
    get name() {
        return this.constructor.name;
    }
    toString() {
        return this.name;
    }
}
exports.Annotation = Annotation;
var annotations;
(function (annotations) {
    class IsTailRec extends Annotation {
    }
    annotations.IsTailRec = IsTailRec;
    class Injected extends Annotation {
    }
    annotations.Injected = Injected;
    class IsOverloaded extends Annotation {
    }
    annotations.IsOverloaded = IsOverloaded;
    class Inline extends Annotation {
    }
    annotations.Inline = Inline;
    class Getter extends Annotation {
    }
    annotations.Getter = Getter;
    class Setter extends Annotation {
    }
    annotations.Setter = Setter;
    class Method extends Annotation {
    }
    annotations.Method = Method;
    class MethodCall extends Annotation {
    }
    annotations.MethodCall = MethodCall;
    class SignatureDeclaration extends Annotation {
    }
    annotations.SignatureDeclaration = SignatureDeclaration;
    class Explicit extends Annotation {
    }
    annotations.Explicit = Explicit;
    class ByPassFunction extends Annotation {
    }
    annotations.ByPassFunction = ByPassFunction;
    class Extern extends Annotation {
        constructor(module, fn) {
            super();
            this.module = module;
            this.fn = fn;
        }
    }
    annotations.Extern = Extern;
    class Export extends Annotation {
        constructor(exportedName) {
            super();
            this.exportedName = exportedName;
        }
    }
    annotations.Export = Export;
    class MutableDeclaration extends Annotation {
    }
    annotations.MutableDeclaration = MutableDeclaration;
    class LabelId extends Annotation {
        constructor(label) {
            super();
            this.label = label;
        }
    }
    annotations.LabelId = LabelId;
    class FunctionInTable extends Annotation {
        constructor(nameIdentifier) {
            super();
            this.nameIdentifier = nameIdentifier;
        }
        toString() {
            return `FunctionInTable(${this.nameIdentifier.name})`;
        }
    }
    annotations.FunctionInTable = FunctionInTable;
    class LocalIdentifier extends Annotation {
        constructor(local) {
            super();
            this.local = local;
        }
        toString() {
            if (this.local instanceof nodes_1.Local) {
                return `Local(${this.local.name}:${this.local.type || '<no-type>'}#${this.local.index})`;
            }
            else {
                return `Global(${this.local.name}:${this.local.type || '<no-type>'})`;
            }
        }
    }
    annotations.LocalIdentifier = LocalIdentifier;
    class IsTailRecCall extends Annotation {
    }
    annotations.IsTailRecCall = IsTailRecCall;
    class CurrentLoop extends Annotation {
        constructor(loop) {
            super();
            this.loop = loop;
            // stub
        }
    }
    annotations.CurrentLoop = CurrentLoop;
    class ImplicitCall extends Annotation {
        constructor(implicitCall) {
            super();
            this.implicitCall = implicitCall;
        }
        toString() {
            return (super.toString() +
                (this.implicitCall.resolvedFunctionType ? this.implicitCall.resolvedFunctionType.inspect(1) : 'not-resolved'));
        }
    }
    annotations.ImplicitCall = ImplicitCall;
    class IsFunctionReference extends Annotation {
        constructor(fun) {
            super();
            this.fun = fun;
        }
    }
    annotations.IsFunctionReference = IsFunctionReference;
    class IsUnreachable extends Annotation {
    }
    annotations.IsUnreachable = IsUnreachable;
    class IsValueNode extends Annotation {
    }
    annotations.IsValueNode = IsValueNode;
    class IsAssignationLHS extends Annotation {
    }
    annotations.IsAssignationLHS = IsAssignationLHS;
    class IsTypeNode extends Annotation {
    }
    annotations.IsTypeNode = IsTypeNode;
    class NoStd extends Annotation {
    }
    annotations.NoStd = NoStd;
    class IsReturnExpression extends Annotation {
        constructor() {
            super(...arguments);
            this.targetLocal = null;
        }
        toString() {
            if (this.targetLocal === null)
                return super.toString();
            return super.toString() + `[${this.targetLocal.index}]`;
        }
    }
    annotations.IsReturnExpression = IsReturnExpression;
})(annotations = exports.annotations || (exports.annotations = {}));
//# sourceMappingURL=annotations.js.map