"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.failWithErrors = exports.failIfErrors = exports.findAllErrors = void 0;
const walker_1 = require("./walker");
const errorPrinter_1 = require("../utils/errorPrinter");
const NodeError_1 = require("./NodeError");
const astPrinter_1 = require("../utils/astPrinter");
const scopePrinter_1 = require("../utils/scopePrinter");
const process = (0, walker_1.walkPreOrder)((token, parsingContext) => {
    if (token.astNode && token.astNode.errors && token.astNode.errors.length) {
        token.astNode.errors.forEach(($) => {
            if ($ && !parsingContext.messageCollector.errors.includes($)) {
                parsingContext.messageCollector.error($);
            }
        });
    }
});
function findAllErrors(document, phase) {
    document.children.forEach($ => process($, phase));
    return document;
}
exports.findAllErrors = findAllErrors;
function failIfErrors(phaseName, document, parsingContext) {
    findAllErrors(document, parsingContext);
    failWithErrors(phaseName, parsingContext);
}
exports.failIfErrors = failIfErrors;
function failWithErrors(phaseName, pc, debug = false) {
    if (!pc.messageCollector.hasErrors())
        return;
    if (pc && pc.messageCollector.errors.length) {
        pc.system.write((0, errorPrinter_1.printErrors)(pc) + '\n');
        if (debug) {
            const hasScopeErrors = pc.messageCollector.errors.some($ => $ instanceof NodeError_1.LysScopeError);
            if (hasScopeErrors) {
                pc.system.write((0, scopePrinter_1.printScopes)(pc) + '\n');
            }
            const modulesWithErrors = new Set();
            pc.messageCollector.errors.forEach($ => {
                const doc = pc.getExistingParsingPhaseForModule($.position.moduleName);
                if (doc) {
                    modulesWithErrors.add(doc);
                }
            });
            const doc = pc.getExistingParsingPhaseForModule('lib::Color');
            if (doc) {
                modulesWithErrors.add(doc);
            }
            modulesWithErrors.forEach($ => {
                pc.system.write($.moduleName + ':\n' + (0, astPrinter_1.printAST)($) + '\n');
            });
            pc.modulesInContext.forEach($ => {
                if (!$.moduleName.includes('#')) {
                    console.dir({
                        module: $.moduleName,
                        analysis: $.analysis,
                        importedBy: $.importedBy,
                        imports: $.importedModules
                    });
                }
            });
            console.log(pc.outline());
        }
    }
    throw Object.assign(new Error(`${phaseName} failed. ${pc.messageCollector.errors.length} errors found:\n` +
        (0, astPrinter_1.indent)(pc.messageCollector.errors
            .map(($, $$) => {
            let msg = $ instanceof NodeError_1.PositionCapableError ? '' + $.message : $.toString() + '\n';
            if (debug && $.stack) {
                msg = msg + '\n' + $.stack;
            }
            return (0, astPrinter_1.indent)(msg, '    ').replace(/^\s+(.*)/m, ($$ + 1).toString() + ')  $1');
        })
            .join('\n'))), { phase: pc });
}
exports.failWithErrors = failWithErrors;
//# sourceMappingURL=findAllErrors.js.map