"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forEachModuleWithPhase = exports.getModuleSet = exports.collectImports = exports.fixParents = exports.getDocument = void 0;
const nodes_1 = require("../nodes");
const NodeError_1 = require("../NodeError");
const walker_1 = require("../walker");
function getDocument(node) {
    let current = node;
    while (current.parent) {
        current = current.parent;
    }
    if (current instanceof nodes_1.Nodes.DocumentNode) {
        return current;
    }
    throw new NodeError_1.LysSemanticError('cannot find parent document', node);
}
exports.getDocument = getDocument;
exports.fixParents = (0, walker_1.walkPreOrder)((node, _, parent) => {
    if (parent) {
        node.parent = parent;
    }
});
/**
 * This function collects the imports of the scopes of a document.
 * Returns true if it added some new import to the Set
 */
function collectImports(moduleList, document, parsingContext) {
    let added = false;
    (0, walker_1.walk)(document, parsingContext, node => {
        if (node.scope && node.scope.importedModules.size) {
            node.scope.importedModules.forEach((_, $) => {
                if (!moduleList.has($)) {
                    added = true;
                    moduleList.add($);
                }
            });
        }
    });
    return added;
}
exports.collectImports = collectImports;
/**
 * Returns a list of all the modules that are necessary to compile the given
 * document.
 *
 * This function can be called only after scope phase has been run
 */
function getModuleSet(document, parsingContext) {
    const moduleList = new Set();
    moduleList.add(document.moduleName);
    collectImports(moduleList, document, parsingContext);
    let added = false;
    do {
        added = false;
        moduleList.forEach($ => {
            try {
                const scope = parsingContext.getPhase($, nodes_1.PhaseFlags.Scope);
                if (scope) {
                    added = collectImports(moduleList, scope, parsingContext);
                }
                else {
                    // ERROR
                }
            }
            catch (e) {
                // ERROR
            }
        });
    } while (added);
    return moduleList;
}
exports.getModuleSet = getModuleSet;
function forEachModuleWithPhase(mainModule, parsingContext, phase, debug = false, cb) {
    const document = parsingContext.getPhase(mainModule, phase, debug);
    const modules = getModuleSet(document, parsingContext);
    modules.forEach($ => {
        const doc = parsingContext.getPhase($, phase, debug);
        if (cb)
            cb(doc);
    });
}
exports.forEachModuleWithPhase = forEachModuleWithPhase;
//# sourceMappingURL=helpers.js.map