import { Nodes } from '../nodes';
import { ParsingContext } from '../ParsingContext';
import { Type } from '../types';
import { Annotation, IAnnotationConstructor } from '../annotations';
import { MessageCollector } from '../MessageCollector';
export declare const MaxAnalysisPassCount = 100;
export declare type LysPropertyDescription = {
    type: Type;
    setter: boolean;
    getter: boolean;
    method: boolean;
};
export declare abstract class TypeResolver {
    documentNode: Nodes.DocumentNode;
    parsingContext: ParsingContext;
    messageCollector: MessageCollector;
    didAnalysisChange: boolean;
    nodeAnnotations: Map<Nodes.Node, Set<Annotation>>;
    private nonResolvedNodes;
    private readonly walk;
    constructor(documentNode: Nodes.DocumentNode, parsingContext: ParsingContext, messageCollector?: MessageCollector);
    analyze(): boolean;
    requiresTypeAnalysis(): boolean;
    protected abstract onLeaveNode(node: Nodes.Node, parsingContext: ParsingContext, parent: Nodes.Node | null): void | boolean;
    protected abstract onEnterNode(_node: Nodes.Node, parsingContext: ParsingContext, parent: Nodes.Node | null): void;
    protected setType(node: Nodes.Node, type: Type | null): void;
    protected removeStashedNodeAnnotation(node: Nodes.Node, annotation: Annotation): boolean;
    protected getStashedNodeAnnotation<T extends Annotation>(node: Nodes.Node, klass: IAnnotationConstructor<T>): T | null;
    protected stashNodeAnnotation(node: Nodes.Node, annotation: Annotation): void;
    protected getType(node: Nodes.Node): Type | null;
    protected setAnalysisChanged(): void;
}
export declare class TypeAnalyzer extends TypeResolver {
    protected onLeaveNode(node: Nodes.Node, _parsingContext: ParsingContext, parent: Nodes.Node): void;
    protected onEnterNode(node: Nodes.Node): void;
    private processMatcherNode;
    private annotateImplicitCall;
    private removeTypeFromMatcherFlow;
    private getTypeTypeType;
    private getTypeFromTypeNode;
    private resolveTypeMember;
    private findFunctionOverload;
    private findImplicitTypeCasting;
    private sizeInBytes;
    private acceptsTypes;
    private downToRefTypes;
    private getTypeSimilarity;
    private canBeAssigned;
    private ensureCanBeAssignedWithImplicitConversion;
    private createImplicitCall;
    private ensureArgumentCoercion;
    private describeProperty;
}
