"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isValidType = void 0;
const types_1 = require("../types");
function isValidType(type) {
    if (type instanceof types_1.IntersectionType || type instanceof types_1.UnionType) {
        return type.of.every(isValidType);
    }
    if (type instanceof types_1.FunctionSignatureType) {
        return isValidType(type.returnType) && type.parameterTypes.every(isValidType);
    }
    if (type instanceof types_1.FunctionType) {
        return isValidType(type.signature);
    }
    if (type instanceof types_1.TypeAlias) {
        return isValidType(type.of);
    }
    if (type instanceof types_1.TraitType) {
        for (let [, value] of type.traitNode.namespaceNames) {
            if (!isValidType(types_1.TypeHelpers.getNodeType(value))) {
                return false;
            }
        }
        return true;
    }
    if (type instanceof types_1.TypeType) {
        return isValidType(type.of);
    }
    if (!type) {
        return false;
    }
    return true;
}
exports.isValidType = isValidType;
//# sourceMappingURL=typeHelpers.js.map