"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.walk = exports.walkPreOrder = void 0;
const NodeError_1 = require("./NodeError");
const nodes_1 = require("./nodes");
function walkPreOrder(cbEnter, cbLeave) {
    const leFn = function (node, phaseResult, parent = null) {
        return walk(node, phaseResult, cbEnter, cbLeave, parent);
    };
    return leFn;
}
exports.walkPreOrder = walkPreOrder;
function walk(node, phaseResult, cbEnter, cbLeave, parent = null) {
    if (node) {
        let traverseIn = true;
        if (cbEnter) {
            try {
                if (false === cbEnter.call(null, node, phaseResult, parent)) {
                    traverseIn = false;
                }
            }
            catch (e) {
                if (phaseResult.messageCollector && e instanceof NodeError_1.PositionCapableError) {
                    phaseResult.messageCollector.error(e);
                }
                else if (phaseResult.messageCollector && node instanceof nodes_1.Nodes.Node) {
                    phaseResult.messageCollector.error(e, node.astNode);
                    console.error(e);
                }
                else
                    throw e;
            }
        }
        if (traverseIn) {
            const children = node.children;
            if (children) {
                for (let i = 0; i < children.length; i++) {
                    if (children[i]) {
                        walk(children[i], phaseResult, cbEnter, cbLeave, node);
                    }
                }
            }
            if (cbLeave) {
                try {
                    cbLeave.call(null, node, phaseResult, parent);
                }
                catch (e) {
                    if (phaseResult.messageCollector && e instanceof NodeError_1.PositionCapableError) {
                        phaseResult.messageCollector.error(e);
                    }
                    else if (phaseResult.messageCollector && node instanceof nodes_1.Nodes.Node) {
                        phaseResult.messageCollector.error(e, node.astNode);
                        console.error(e);
                    }
                    else
                        throw e;
                }
            }
        }
    }
}
exports.walk = walk;
//# sourceMappingURL=walker.js.map