"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSystem = void 0;
const path_1 = require("path");
class WebSystem {
    constructor() {
        this.args = [];
        this.newLine = '\n';
        this.useCaseSensitiveFileNames = true;
        this.files = new Map();
    }
    write(s) {
        console.log(s);
    }
    writeOutputIsTTY() {
        return false;
    }
    readFile(path) {
        const file = this.files.get(this.resolvePath(path));
        if (file) {
            return file.content;
        }
        return undefined;
    }
    getFileSize(path) {
        const file = this.files.get(this.resolvePath(path));
        if (file) {
            return file.content.length;
        }
        return 0;
    }
    writeFile(path, data, _writeByteOrderMark) {
        const file = this.files.get(this.resolvePath(path));
        if (file) {
            file.content = data;
        }
        else {
            this.files.set(this.resolvePath(path), {
                content: data
            });
        }
        return;
    }
    resolvePath(...path) {
        return (0, path_1.resolve)(...path);
    }
    relative(from, to) {
        return (0, path_1.relative)(from, to);
    }
    fileExists(path) {
        return this.files.has(this.resolvePath(path));
    }
    directoryExists(path) {
        for (let [key] of this.files) {
            if (key.startsWith(this.resolvePath(path)))
                return true;
        }
        return false;
    }
    createDirectory() {
        // stub
    }
    getCurrentDirectory() {
        return '/';
    }
}
exports.WebSystem = WebSystem;
//# sourceMappingURL=WebSystem.js.map