"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringCodeWriter = void 0;
class StringCodeWriter {
    constructor(indentSpaces = 2) {
        this.indentSpaces = indentSpaces;
        this._indent = 0;
        this._content = '';
        this._newLineCallBacks = [];
        // stub
    }
    println(code) {
        if (code !== undefined) {
            this._content = this._content + code;
            this.println();
        }
        else {
            this.callNewLineCallbacks();
            this._content = this._content + '\n';
        }
        return this;
    }
    indent() {
        this._indent += 1;
        return this;
    }
    printSpace(code = '') {
        return this.append(code).append(' ');
    }
    printIndent() {
        this._content =
            this._content +
                Array(this._indent * this.indentSpaces)
                    .fill(' ')
                    .join('');
        return this;
    }
    print(code) {
        return this.append(code);
    }
    printQuoted(text) {
        this.append('"')
            .append(text)
            .append('"');
        return this;
    }
    printForeachWithSeparator(separator, elements, code) {
        elements.forEach((element, index) => {
            if (index >= 1) {
                this.print(separator);
            }
            code(element);
        });
        return this;
    }
    dedent() {
        this._indent -= 1;
        return this;
    }
    onNewLine(callback) {
        this._newLineCallBacks.push(callback);
        return this;
    }
    hasNewLineListener() {
        return this._newLineCallBacks.length !== 0;
    }
    codeContent() {
        return this._content;
    }
    toString() {
        return this.codeContent();
    }
    append(code) {
        if (this._content && this._content.endsWith('\n')) {
            this.printIndent();
        }
        this._content = this._content + code;
        return this;
    }
    callNewLineCallbacks() {
        const buffer = this._newLineCallBacks.slice();
        this._newLineCallBacks.length = 0;
        buffer.forEach(callback => callback(this));
    }
}
exports.StringCodeWriter = StringCodeWriter;
//# sourceMappingURL=StringCodeWriter.js.map