"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.indent = exports.printAST = void 0;
const nodes_1 = require("../compiler/nodes");
const types_1 = require("../compiler/types");
function printAST(token, level = 0, printedNodes = new Set()) {
    if (!token)
        return '<no token was provided>';
    if (printedNodes.has(token)) {
        return '  '.repeat(level) + `|-${token.nodeName}$- THIS NODE HAS BEEN PRINTED`;
    }
    printedNodes.add(token);
    const nodeType = types_1.TypeHelpers.getNodeType(token);
    const ofType = nodeType ? ' type=' + nodeType.inspect(10) : '';
    let text = '';
    const resolvedReference = token instanceof nodes_1.Nodes.ReferenceNode
        ? ' resolvedReference=' + (token.resolvedReference ? token.resolvedReference.toString() : '<null>')
        : '';
    if (token instanceof nodes_1.Nodes.QNameNode) {
        text = '=' + token.text.replace(/\n/g, '\\n');
    }
    else if (token instanceof nodes_1.Nodes.NameIdentifierNode) {
        text = '=' + JSON.stringify(token.name);
    }
    else if (token instanceof nodes_1.Nodes.IntegerLiteral ||
        token instanceof nodes_1.Nodes.HexLiteral ||
        token instanceof nodes_1.Nodes.FloatLiteral) {
        text =
            '=' +
                JSON.stringify(token.value) +
                (token.suffixReference && token.suffixReference.variable ? token.suffixReference.variable.text : '');
    }
    else if (token instanceof nodes_1.Nodes.LiteralNode) {
        text = '=' + JSON.stringify(token.value);
    }
    else if (token instanceof nodes_1.Nodes.AbstractFunctionCallNode) {
        text = token.resolvedFunctionType ? '=' + token.resolvedFunctionType.inspect(10) : '';
    }
    const annotations = token.getAnnotations().length > 0
        ? ' annotations=' +
            token
                .getAnnotations()
                .map($ => $.toString())
                .join(',')
        : '';
    return ('\n' +
        '  '.repeat(level) +
        `|-${token.nodeName}${text}${resolvedReference}${ofType}${annotations}` +
        (token.children || []).map(c => printAST(c, level + 1)).join(''));
}
exports.printAST = printAST;
function indent(str, indentation = '  ') {
    if (!str.replace) {
        console.trace();
    }
    return str.replace(/^(.*)$/gm, indentation + '$1').replace(/^\s+$/gm, '');
}
exports.indent = indent;
//# sourceMappingURL=astPrinter.js.map