"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTestResults = void 0;
const execution_1 = require("../execution");
const colors_1 = require("../colors");
function writeLn(message) {
    process.stderr.write(message);
    process.stderr.write('\n');
}
function getTestResults(instance) {
    return instance.testResults || (instance.testResults = []);
}
exports.getTestResults = getTestResults;
function default_1(getInstance) {
    function getTestStack() {
        const instance = getInstance();
        return instance.testStack || (instance.testStack = []);
    }
    function getTopTest() {
        const testStack = getTestStack();
        if (testStack.length) {
            return testStack[testStack.length - 1];
        }
        return null;
    }
    return {
        test: {
            printMemory: (start, length) => {
                writeLn(`Dump from ${start.toString(16)} of ${length} bytes`);
                while (start % 16 !== 0 && start !== 0) {
                    // tslint:disable-next-line:no-parameter-reassignment
                    start--;
                }
                while (length % 16 !== 0) {
                    // tslint:disable-next-line:no-parameter-reassignment
                    length++;
                }
                writeLn((0, execution_1.hexDump)(getInstance().exports.memory.buffer, start + length, start));
            },
            pushTest: (offset) => {
                const title = (0, execution_1.readString)(getInstance().exports.memory.buffer, offset);
                const top = getTopTest();
                const suite = {
                    title,
                    startTime: Date.now(),
                    passed: true,
                    skipped: false,
                    endTime: Date.now()
                };
                if (top) {
                    (top.tests = top.tests || []).push(suite);
                }
                else {
                    getTestResults(getInstance()).push(suite);
                }
                getTestStack().push(suite);
                const level = getTestStack().length;
                const indentation = '  '.repeat(level);
                writeLn('\n' + indentation + title + ':');
            },
            popTest: () => {
                if (getTestStack().length) {
                    const suite = getTestStack().pop();
                    if (suite) {
                        if ((suite.tests && suite.tests.some($ => !$.passed)) ||
                            (suite.assertions && suite.assertions.some($ => !$.passed))) {
                            suite.passed = false;
                        }
                        suite.endTime = Date.now();
                    }
                }
            },
            registerAssertion: (passed, nameOffset) => {
                const title = (0, execution_1.readString)(getInstance().exports.memory.buffer, nameOffset);
                const didPass = !!passed;
                const suite = getTopTest();
                if (suite) {
                    suite.passed = suite.passed && didPass;
                    suite.assertions = suite.assertions || [];
                    suite.assertions.push({
                        title,
                        passed: didPass
                    });
                }
                const level = getTestStack().length;
                const indentation = '  '.repeat(level + 1);
                if (didPass) {
                    writeLn(indentation +
                        (0, colors_1.formatColorAndReset)('✓ ', colors_1.ForegroundColors.Green) +
                        (0, colors_1.formatColorAndReset)(title, colors_1.ForegroundColors.Grey));
                }
                else {
                    writeLn(indentation +
                        (0, colors_1.formatColorAndReset)('X ', colors_1.ForegroundColors.Red) +
                        (0, colors_1.formatColorAndReset)(title, colors_1.ForegroundColors.Grey));
                }
            },
            printNumber: (x) => {
                writeLn('printNumber: ' + x);
            },
            printString: (offset) => {
                const str = (0, execution_1.readString)(getInstance().exports.memory.buffer, offset);
                writeLn(str);
            }
        }
    };
}
exports.default = default_1;
//# sourceMappingURL=test.js.map