"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadFromMD = void 0;
const MDParser_1 = require("./MDParser");
const errorPrinter_1 = require("./errorPrinter");
const annotations_1 = require("../compiler/annotations");
function loadFromMD(parsingContext, mdContent) {
    const parsedMD = (0, MDParser_1.parseMD)(mdContent);
    parsingContext.reset();
    let mainModule = 'main';
    const jsFiles = {};
    const lysFiles = {};
    parsedMD.forEach($ => {
        if ($.type === 'file') {
            if ($.fileName.text.endsWith('.lys')) {
                const moduleName = parsingContext.getModuleFQNForFile($.fileName.text);
                parsingContext.invalidateModule(moduleName);
                lysFiles[$.fileName.text] = moduleName;
                const document = parsingContext.getParsingPhaseForContent($.fileName.text, moduleName, $.codeBlock.text);
                if ($.codeBlock.text.includes(`#![no-std]`)) {
                    document.annotate(new annotations_1.annotations.NoStd());
                }
            }
            if ($.fileName.text.endsWith('.js')) {
                // tslint:disable-next-line:no-eval
                jsFiles[$.fileName.text] = eval($.codeBlock.text);
                if (typeof jsFiles[$.fileName.text] !== 'function') {
                    throw new errorPrinter_1.LysError(`The file ${JSON.stringify($.fileName.text)} is not a function`);
                }
            }
        }
    });
    {
        let mainPresent = false;
        for (let i in lysFiles) {
            if (lysFiles[i] === mainModule) {
                mainPresent = true;
                break;
            }
        }
        if (!mainPresent) {
            throw new errorPrinter_1.LysError(`The module ${JSON.stringify(mainModule)} is not present in the Markdown file, got: ${Object.values(lysFiles)}`);
        }
    }
    return {
        jsFiles,
        lysFiles,
        mainModule
    };
}
exports.loadFromMD = loadFromMD;
//# sourceMappingURL=loadFromMD.js.map